#------------------------------------------------------------------------------>--
# File: Parse_results_str_est.R
# Description: Parses results from the structural estimation from Matlab
# Author: Vladimir Yankov (vladimir.yankov@gmail.com)
#------------------------------------------------------------------------------>--
# Clear workspace
rm(list = ls())
# Load required libaries
final_data_dir<- ".Data_InSearchOfRiskFreeAsset/str_est/"
lib_loc = paste0(root_dir, "Code/R/Packages/")
figures_dir = paste0(root_dir,"./Results/Figures/")
code_proj_dir = paste0(root_dir,"./StructuralEstimation/functions/")
#------------------------------------------------------------------------------>--
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW","doParallel",
               "sandwich","stargazer","neldermead",
               "nleqslv","DEoptim","R.matlab","latex2exp",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}
# Set multi-threading
setMKLthreads(18)
#------------------------------------------------------------------------------>--
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
sigma = 1.225
beta = 0.99
tau = 1

# marginal propensity to consume
h = function(R, sigma, beta, tau){
  if(sigma==1){
    hh = rep(1/(1+beta^tau), length(R))
    return(hh)
  }
  if(sigma!=1){
    return(1/(1+beta^(tau*sigma)*R^(sigma-1)))
  }
}
# marginal utility of wealth
phi = function(R, sigma, beta, tau){
  if(sigma !=1){
    phi = h(R, sigma, beta, tau)^(1/(1-sigma))
  }
  if(sigma ==1){
    phi = log(h(R, sigma, beta, tau)) + (beta^tau)*log((1-h(R, sigma, beta, tau))*R)
  }
  return(phi)
}

## first derivative of the marginal utility of wealth
phi_prime = function(R, sigma, beta, tau){
  if(sigma !=1){
    phi_prime = phi(R, sigma, beta, tau)*(1-h(R, sigma, beta, tau))/R
    return(phi_prime)
  }
}


#------------------------------------------------------------------------------>--
# Load data ----
#------------------------------------------------------------------------------>--
setwd(paste0(data_dir,"RW_data/RW_final/deposits/"))
load(file="deposits_msa.rda")
load(file="deposits_msa_m.rda")
load("msa_code.rda")
load(file=paste0(sod_dir,"sumd_msa_market.rda"))
sumd_msa$msa_code=sumd_msa$msa_code_close
sumd_msa$msa_code_close=NULL
dates = unique(deposits_msa_m$date)
deposits_msa_agg_t = deposits_msa_m[, list(n_br_loc = sum(n_br_loc,na.rm=T),
                                           dep_msa = sum(dep_msa,na.rm=T),
                                           nbanks = length(unique(id_rssd_hd_off)),
                                           nbhc = length(unique(id_rssd_top))),
                                    by = c("date","msa_code") ]
deposits_msa_agg_t = deposits_msa_agg_t[order(deposits_msa_agg_t$msa_code,
                                              deposits_msa_agg_t$date),]
deposits_msa_agg = deposits_msa_agg_t[,list(nbanks_min = min(nbanks, na.rm=T),
                                            nbanks_max = max(nbanks, na.rm=T),
                                            nbanks_sd = sd(nbanks, na.rm=T),
                                            dep_last = dep_msa[date==max(date)]/10^6,
                                            dstart = min(date),
                                            dend = max(date)),
                                      by = "msa_code"]
deposits_msa_agg = merge(deposits_msa_agg,msa_code, by ="msa_code")
deposits_msa_agg = deposits_msa_agg[order(-deposits_msa_agg$nbanks_min),]
deposits_msa_agg[,total_deposits:= sum(dep_last,na.rm=T)]
msa_codes_to_estimate = deposits_msa_agg$msa_code[deposits_msa_agg$nbanks_max>20]


#------------------------------------------------------------------------------>--
# ___ Load structural estimates----
#------------------------------------------------------------------------------>--
setwd(final_data_dir)
msa_est = list.files(pattern = "results_msa_")
msa_est = data.frame(file = as.character(msa_est),
                     msa_code = as.numeric(substr(msa_est,13,17)),
                     date_est = substr(msa_est,19,29))
msa_est$date_est=as.Date(msa_est$date_est,"%d_%b_%Y")
msa_est=data.table(msa_est)
msa_est = msa_est[, list(file = file[date_est == max(date_est)]),
                  by = "msa_code"]
msa_est = merge(msa_est,deposits_msa_agg, by = "msa_code")
msa_est[,total_deposits_est := sum(dep_last,na.rm=T)]
msa_est[,share_total := 100*total_deposits_est/total_deposits]
msa_codes_est = unique(msa_est$msa_code)
msa_codes_to_estimate = msa_codes_to_estimate[!msa_codes_to_estimate %in% msa_codes_est]
msa_est_remaining = deposits_msa_agg[deposits_msa_agg$msa_code %in% msa_codes_to_estimate,]
write.csv(msa_est_remaining,file = "msa_est_remaining.csv")
# ----------------------------------------------------------------------------->
# Extract qvec estimates from mat files----
# ----------------------------------------------------------------------------->
cluster = makeCluster(20, type = "SOCK")
#registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#------------------------------------------------------------------------------>
msa_est$file =as.character(msa_est$file)
qvec_msa_m = foreach(m = msa_est$file, .combine = "rbind", .errorhandling = 'remove',
                     .packages = c("foreach","data.table","R.matlab"))%do%{
  print(m)
  mat_file = readMat(m)
  mat_file=mat_file$res.est

  est_ids = dim(mat_file)[3]

  qvec = foreach(t = 1:est_ids, .combine ="rbind", .errorhandling = 'remove')%do%{
   #print(t)
    # "msa"       "sigma"     "qvec.est"  "q1"        "qN"
    # "delta.k"   "xi.vec"    "F.xi"      "ks.pvalue"
    # "R.t"       "R.max"     "R.min"     "R.sd"      "date"
    # "nbanks"    "exit"      "fval"      "t.est"
    est_t = mat_file[,1,t]

    est_data = data.frame(msa_code = est_t$msa,
                          date = as.Date(est_t$date,"%d-%b-%Y"),
                          qvec = round(est_t$qvec.est,4),
                          nbanks = est_t$nbanks,
                          R_sd= est_t$R.sd,
                          R_t = est_t$R.t,
                          R_max = est_t$R.max,
                          R_range = est_t$R.max-est_t$R.min,
                          ks_p = est_t$ks.pvalue,
                          exit =  est_t$exit)

    est_data = data.table(est_data)
    est_data[,q_k := seq(.N)]
    est_data = data.frame(est_data)

    return(est_data)
  }
  return(qvec)
}
rm(qvec)
#------------------------------------------------------------------------------>
# aggregate estimates:
qvec_msa_m=data.table(qvec_msa_m)
# add total deposits by MSA area
qvec_msa_m = merge(qvec_msa_m,deposits_msa_agg_t[,list(msa_code,date,dep_msa)],
                   by = c("date", "msa_code"))
qvec_msa_m[,year:=year(date)]
# Constructs
qvec_msa_m[, qsum:=sum(qvec*q_k), by = c("msa_code","date")]
qvec_msa_m[, search_avg := qsum/nbanks]
qvec_msa_m[, qN:=qvec[q_k==nbanks], by = c("msa_code","date")]
qvec_msa_m[, q1:=qvec[q_k==1], by = c("msa_code","date")]
qvec_msa_m[, q2:=qvec[q_k==2], by = c("msa_code","date")]
qvec_msa_m[, q3:=qvec[q_k==3], by = c("msa_code","date")]
qvec_msa_m[, year:=year(date)]
qvec_msa_y = qvec_msa_m[,list(q1 = mean(q1,na.rm=T),
                              q2 = mean(q2,na.rm=T),
                              q3 = mean(q3,na.rm=T),
                              qN = mean(qN,na.rm=T),
                              nbanks = median(nbanks,na.rm=T),
                              R_sd = mean(R_sd,na.rm=T),
                              R_t = mean(R_t,na.rm =T),
                              R_max = mean(R_max,na.rm=T),
                              R_range = mean(R_range,na.rm=T),
                              search_avg = mean(search_avg,na.rm=T)),
                        by = c("msa_code","year")]
qvec_msa_y=merge(qvec_msa_y,sumd_msa,by =c("msa_code","year"),all.x=T )
qvec_msa_y[,rw_coverage := 100*nbanks/n_banks_msa]
# Collapse/aggregate by date
qvec_agg_m = qvec_msa_m[,
                        list(q1 = mean(q1,na.rm=T),
                              q1_p5 = quantile(q1,probs=0.05,na.rm=T),
                              q1_p50 = quantile(q1,probs=0.5,na.rm=T),
                              q1_p95 = quantile(q1,probs=0.95,na.rm=T),
                              q1_w = weighted.mean(q1,w=dep_msa,na.rm=T),
                              q2 = mean(q2,na.rm=T),
                              q2_w = weighted.mean(q2,w=dep_msa,na.rm=T),
                              q3 = mean(q3,na.rm=T),
                              q3_w = weighted.mean(q3,w=dep_msa,na.rm=T),
                              qN = mean(qN,na.rm=T),
                              qN_w = weighted.mean(qN,w=dep_msa,na.rm=T),
                              R_sd = mean(R_sd,na.rm=T),
                              R_sd_w = weighted.mean(R_sd,w=dep_msa,na.rm=T),
                              R_t = mean(R_t,na.rm =T),
                              R_t_w = weighted.mean(R_t,w=dep_msa,na.rm=T),

                              R_t_p5 = quantile(R_t,probs=0.05,na.rm=T),
                              R_t_p50 = quantile(R_t,probs=0.5,na.rm=T),
                              R_t_p95 = quantile(R_t,probs=0.95,na.rm=T),

                              R_max = mean(R_max,na.rm=T),
                              R_max_w = weighted.mean(R_max,w=dep_msa,na.rm=T),
                              R_range = mean(R_range,na.rm=T),
                              R_range_w = weighted.mean(R_range,w=dep_msa,na.rm=T),
                              search_avg = mean(search_avg,na.rm=T),
                              search_p5 = quantile(search_avg,probs=0.05,na.rm=T),
                              search_p50 = quantile(search_avg,probs=0.5,na.rm=T),
                              search_p95 = quantile(search_avg,probs=0.95,na.rm=T),
                              search_avg_w = weighted.mean(search_avg,w=dep_msa,na.rm=T)),
                        by = "date"]
qvec_agg_m = qvec_agg_m[order(qvec_agg_m$date),]

qvec_agg_y = qvec_msa_m[,list(q1 = mean(q1,na.rm=T),
                              q1_w = weighted.mean(q1,w=dep_msa,na.rm=T),
                              q1_p5 = quantile(q1,probs=0.05,na.rm=T),
                              q1_p50 = quantile(q1,probs=0.5,na.rm=T),
                              q1_p95 = quantile(q1,probs=0.95,na.rm=T),
                              q2 = mean(q2,na.rm=T),
                              q2_w = weighted.mean(q2,w=dep_msa,na.rm=T),
                              q3 = mean(q3,na.rm=T),
                              q3_w = weighted.mean(q3,w=dep_msa,na.rm=T),
                              qN = mean(qN,na.rm=T),
                              qN_w = weighted.mean(qN,w=dep_msa,na.rm=T),
                              R_sd = mean(R_sd,na.rm=T),
                              R_sd_w = weighted.mean(R_sd,w=dep_msa,na.rm=T),
                              R_t = mean(R_t,na.rm =T),
                              R_t_w = weighted.mean(R_t,w=dep_msa,na.rm=T),
                              R_max = mean(R_max,na.rm=T),
                              R_max_w = weighted.mean(R_max,w=dep_msa,na.rm=T),
                              R_range = mean(R_range,na.rm=T),
                              R_range_w = weighted.mean(R_range,w=dep_msa,na.rm=T),
                              search_avg = mean(search_avg,na.rm=T),
                              search_p5 = quantile(search_avg,probs=0.05,na.rm=T),
                              search_p50 = quantile(search_avg,probs=0.5,na.rm=T),
                              search_p95 = quantile(search_avg,probs=0.95,na.rm=T),
                              search_avg_w = weighted.mean(search_avg,w=dep_msa,na.rm=T)),
                        by = "year"]

qvec_agg_y = qvec_agg_y[order(qvec_agg_y$year),]
qvec_agg_m = qvec_agg_m[order(qvec_agg_m$date),]

save(qvec_msa_m,qvec_msa_y,qvec_agg_y,qvec_agg_m,file = "qvec_est_msa.rda")


#------------------------------------------------------------------------------>
# Estimates of the search cost distribution----
#------------------------------------------------------------------------------>
F_xi_msa_m = foreach(m = msa_est$file, .combine = "rbind", .errorhandling = 'remove')%do%{
  print(m)
  mat_file = readMat(m)
  mat_file=mat_file$res.est
  est_ids = dim(mat_file)[3]
  xi = foreach(t = 1:est_ids, .combine ="rbind", .errorhandling = 'remove')%do%{
    #print(t)
    # "msa"       "sigma"     "qvec.est"  "q1"        "qN"
    # "delta.k"   "xi.vec"    "F.xi"      "ks.pvalue"
    # "R.t"       "R.max"     "R.min"     "R.sd"      "date"
    # "nbanks"    "exit"      "fval"      "t.est"
    est_t = mat_file[,1,t]

    est_data = data.frame(msa_code = est_t$msa,
                          date = as.Date(est_t$date,"%d-%b-%Y"),
                          xi = est_t$xi.vec,
                          deltak = est_t$delta.k,
                          Fxi = est_t$F.xi,
                          nbanks = est_t$nbanks,
                          R_sd= est_t$R.sd,
                          R_min = est_t$R.min,
                          R_max = est_t$R.max,
                          R_t = est_t$R.t,
                          R_range = est_t$R.max-est_t$R.min,
                          ks_p = est_t$ks.pvalue,
                          exit =  est_t$exit)

    est_data = data.table(est_data)
    est_data[,q_k := seq(.N)]
    est_data = data.frame(est_data)

    return(est_data)
  }
  return(xi)
}
F_xi_msa_m = data.table(F_xi_msa_m)
F_xi_msa_m[,delta1 := deltak[q_k==1], by = c("msa_code","date")]
F_xi_msa_m[,delta2 := deltak[q_k==2], by = c("msa_code","date")]
F_xi_msa_m[,R_min:=R_max-R_range, by = c("msa_code","date")]
F_xi_msa_m[,dR_d1 := delta1/phi_prime((R_min+R_range)/2,sigma,beta,tau), by = c("msa_code","date")]
F_xi_msa_m[,dRmin_d1 := delta1/phi_prime(R_min,sigma,beta,tau), by = c("msa_code","date")]
F_xi_msa_m = merge(F_xi_msa_m,deposits_msa_agg_t[,list(msa_code,date,dep_msa)],
                   by = c("date", "msa_code"))
F_xi_agg_m = F_xi_msa_m[,list(delta1 = mean(delta1,na.rm=T),
                              delta1_w = weighted.mean(delta1,w=dep_msa,na.rm=T),

                              delta1_p5 = quantile(delta1,probs=0.05,na.rm=T),
                              delta1_p50 = quantile(delta1,probs=0.5,na.rm=T),
                              delta1_p95 = quantile(delta1,probs=0.95,na.rm=T),

                              dR_d1 = mean(dR_d1,na.rm=T),
                              dR_d1_w = weighted.mean(dR_d1,w=dep_msa,na.rm=T),
                              dR_d1_p5 = quantile(dR_d1,probs=0.05,na.rm=T),
                              dR_d1_p50 = quantile(dR_d1,probs=0.5,na.rm=T),
                              dR_d1_p95 = quantile(dR_d1,probs=0.95,na.rm=T),


                              dRmin_d1 = mean(dRmin_d1,na.rm=T),
                              dRmin_d1_w = weighted.mean(dRmin_d1,w=dep_msa,na.rm=T),
                              dRmin_d1_p5 = quantile(dRmin_d1,probs=0.05,na.rm=T),
                              dRmin_d1_p50 = quantile(dRmin_d1,probs=0.5,na.rm=T),
                              dRmin_d1_p95 = quantile(dRmin_d1,probs=0.95,na.rm=T),

                              delta2 = mean(delta2,na.rm=T),
                              delta2_w = weighted.mean(delta2,w=dep_msa,na.rm=T)
                              ), by = "date"]
F_xi_agg_m = F_xi_agg_m[order(F_xi_agg_m$date),]
#++++ Save (F_xi_msa_m)----
save(F_xi_msa_m,F_xi_agg_m, msa_est,file = "F_xi_msa_m.rda")

#-------------------------------------------------------------------------------->
## Search cost distribution F(delta)----
#-------------------------------------------------------------------------------->
# Select MSA markets with p-values exceeding 10 percent
F_xi_msa_m=F_xi_msa_m[F_xi_msa_m$ks_p>0.05]

F_xi_msa_m[,xi_Rmin:=10^4*xi/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27),
           by = c("msa_code","date")]
F_xi_msa_m[,deltak_Rmin:=10^4*deltak/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27),
           by = c("msa_code","date")]
#F_xi_msa_m = F_xi_msa_m[F_xi_msa_m$ks_p>0.05]
F_xi_msa_m = F_xi_msa_m[order(F_xi_msa_m$msa_code,
                              F_xi_msa_m$date),]
datem = unique(F_xi_msa_m$date)
datem = datem[order(datem)]
length(unique(F_xi_msa_m$msa_code))

#-------------------------------------------------------------------------------->
cluster = makeCluster(20, type = "SOCK")
#registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#-------------------------------------------------------------------------------->

F_xi = foreach(d =datem, .combine ="rbind",
               .packages = c("foreach","lubridate","data.table"),
               .verbose = T,
               .export = c("phi_prime"))%dopar%{
                 print(d)
                 data_d = F_xi_msa_m[F_xi_msa_m$date==d,]
                 data_d= data_d[order(-data_d$dep_msa),]
                 msa_d = unique(data_d$msa_code) # unique MSA

                 delta1_Rmin_min_d = min(data_d$deltak_Rmin[data_d$q_k==1]) # minimum delta_1
                 xi_Rmin_max_d = max(data_d$xi_Rmin) # maximum delta_1

                 xi_min = min(data_d$xi)
                 xi_max = max(data_d$xi)

                 xi_grid =  seq(0,1.5*xi_max,
                                length.out = 1000) # grid for xi

                 xi_Rmin_grid_d = seq(0,1.5*xi_Rmin_max_d,
                                      length.out = 1000) # grid for xi_Rmin




                 F_xi_d = foreach(m =msa_d, .combine ="rbind",
                                  .packages = c("foreach","lubridate"),
                                  .export = c("phi_prime"))%do%{
                                    print(m)
                                    data_m = data_d[data_d$msa_code==m,]
                                    dep_msa = unique(data_m$dep_msa)
                                    qvec = qvec_msa_m$qvec[qvec_msa_m$date==d & qvec_msa_m$msa_code==m]
                                    nbanks = length(qvec)
                                    delta =data_m$deltak
                                    delta_Rmin = 10^4*data_m$deltak/phi_prime(R=data_m$R_min,beta=0.99,tau=1, sigma=1.27)

                                    xi_Rmin = rev(delta_Rmin[1:(nbanks-1)])
                                    xi = data_m$xi
                                    xi = xi[2:length(xi)]


                                    Fxi = rev(1-cumsum(qvec[1:(nbanks-1)]))
                                    Fxi_fit = splinefun(xi_Rmin, Fxi, method = "monoH.FC")
                                    Fxi_fit_xi = splinefun(xi, Fxi, method = "monoH.FC")


                                    # impose that Fxi in (0,1)
                                    Fxi = Fxi_fit(xi_Rmin_grid_d)
                                    Fxi_xi = Fxi_fit_xi(xi_grid)

                                    if(length(which(Fxi_xi>1))>0){Fxi_xi[which(Fxi_xi>1)]=1}
                                    if(length(which(Fxi_xi<0))>0){Fxi_xi[which(Fxi_xi<0)]=0}

                                    if(length(which(Fxi>1))>0){Fxi[which(Fxi>1)]=1}
                                    if(length(which(Fxi<0))>0){Fxi[which(Fxi<0)]=0}

                                    Fxi_m = data.frame(xi_Rmin = xi_Rmin_grid_d,
                                                       xi = xi_grid,
                                                       Fxi = Fxi,
                                                       Fxi_xi,
                                                       delta1_Rmin = max(delta_Rmin),
                                                       delta2_Rmin = delta_Rmin[2],
                                                       delta1 = max(xi),
                                                       dep_msa = dep_msa,
                                                       msa_code = m,
                                                       date = d)
                                    return(Fxi_m)}

                 # aggregate across markets
                 F_xi_agg = data.table(F_xi_d)
                 F_xi_agg = F_xi_agg[,list(nmarkets = length(unique(msa_code)),
                                           xi_max = xi_max,
                                           xi_min = xi_min,
                                           xi_Rmin_max = xi_Rmin_max_d,
                                           delta1_Rmin_min_d = delta1_Rmin_min_d,
                                           delta1_Rmin_wm = weighted.mean(delta1_Rmin,w=dep_msa,na.rm=T),
                                           delta2_Rmin_wm = weighted.mean(delta2_Rmin,w=dep_msa,na.rm=T),
                                           delta1_wm = weighted.mean(delta1,w=dep_msa,na.rm=T),
                                           Fxi_xi_p5 = quantile(Fxi_xi,probs = 0.05),
                                           Fxi_xi_p50 = quantile(Fxi_xi,probs = 0.50),
                                           Fxi_xi_p95 = quantile(Fxi_xi,probs = 0.95),
                                           Fxi_xi_wm = weighted.mean(Fxi_xi,w=dep_msa),
                                           Fxi_p5 = quantile(Fxi,probs = 0.05),
                                           Fxi_p50 = quantile(Fxi,probs = 0.50),
                                           Fxi_p95 = quantile(Fxi,probs = 0.95),
                                           Fxi_wm = weighted.mean(Fxi,w=dep_msa)),
                                     by =c("xi_Rmin","xi","date")]
                 F_xi_agg$xi_Rmin_median =  F_xi_agg$xi_Rmin[which.min(abs(F_xi_agg$Fxi_wm-0.5))]
                 F_xi_agg$xi_median =  F_xi_agg$xi[which.min(abs(F_xi_agg$Fxi_xi_wm-0.5))]



                 # compute the variance
                 Fxi_fit = with(F_xi_agg,
                                splinefun(xi_Rmin, Fxi_wm, method = "monoH.FC"))
                 Fxi_xi_fit = with(F_xi_agg,
                                   splinefun(xi, Fxi_xi_wm, method = "monoH.FC"))

                 Fxi_fit = Vectorize(Fxi_fit)
                 Fxi_xi_fit = Vectorize(Fxi_xi_fit)

                 Fxi_fit2 = function(x){
                   y = Fxi_fit(x)

                   for(i in 1:length(y)){
                     if(y[i]>1){y[i]=1}
                     if(y[i]<0){y[i]=0}
                   }
                   return(y)}
                 int_fun1 = function(x){ x*(1- Fxi_fit2(x))}
                 int_fun2 = function(x){ (1- Fxi_fit2(x))}


                 Fxi_xi_fit2 = function(x){
                   y = Fxi_xi_fit(x)

                   for(i in 1:length(y)){
                     if(y[i]>1){y[i]=1}
                     if(y[i]<0){y[i]=0}
                   }
                   return(y)}
                 int_xi_fun1 = function(x){ x*(1- Fxi_xi_fit2(x))}
                 int_xi_fun2 = function(x){ (1- Fxi_xi_fit2(x))}



                 xi_sd = sqrt(2*integrate(int_fun1,lower = 0, upper =Inf )$value-
                                integrate(int_fun2,lower = 0, upper = Inf)$value^2)

                 xi_sd2 = sqrt(2*integrate(int_xi_fun1,lower = 0, upper =Inf )$value-
                                 integrate(int_xi_fun2,lower = 0, upper = Inf)$value^2)

                 F_xi_agg$xi_Rmin_sd=xi_sd
                 F_xi_agg$xi_sd=xi_sd2
                 return(F_xi_agg)
               }
stopCluster(cluster)
save(F_xi,file =paste0(final_data_dir,"F_xi.rda"))


#-------------------------------------------------------------------------------->

