%--------------------------------------------------------------------------
% SearchCostEstimation_sigma.m
% Author: Vladimir Yankov (vladimir.yankov@gmail.com)
% Description: Estimates the search cost distributions
% Codes provided for replication purposes for 
% "In Search of a Risk-free Asset" forthcoming at the JMCB
% The code requires data on deposit rates from RateWatch (RW) constructed with 
% a set of R codes provided in the package. The weekly RW offer rates are
% organized in monthly data in deposits_msa_m.csv.
% msa_code.csv contains the msa codes and the names of MSA areas
% deposits_msa_agg.csv contains MSA level information such as deposit
% amounts, number of banks. The code estimates search costs for the largest
% MSA areas first (sorts by deposit size).
% To speed computation, I use Matlabs parallel optimization toolbox. I use
% 32 cores but the user can specify the number depending on the capacity of
% their computer.
% Results from the estimation are stored for each MSA area and are then 
% parsed with R code into datasets used for analysis. Note that the
% estimaton is time consuming especially for markets with larger number of
% banks.The code contains two ways of computing the scores of the
% likelihood. One is based on the symbolic toolbox of Matlab. The second is
% based on the exact analytic derivatives presented in the online appendix.
% The code uses the analytic derivatives, which significantly speeds up the
% computation.
% See Parse_results_str_est.R this code reads the results from the
% estimation and constructs a panel dataset with search cost estimates by
% MSA area and time.
%--------------------------------------------------------------------------
%%
clear
close all
clc
% folders
rw_dir = './RW_final/deposits/';   % underlying RW data
functions_dir = './functions/';    % functions
results_dir = './str_est/';        % directory to store end results
% Add path to the functions
path(path,functions_dir);
%%--------------------------------------------------------------------------
cd(functions_dir)
% Load data
%--------------------------------------------------------------------------
tic;
% Load deposit dataset
deposits = readtable([rw_dir,'deposits_msa_m.csv']);
msa_code = readtable([rw_dir,'msa_code.csv']);
deposits_msa_info = readtable([rw_dir,'deposits_msa_agg.csv']);
deposits_msa_info = sortrows(deposits_msa_info,{'dep_last','nbanks_max'}, 'descend');

% identify msa_codes and dates 
msa_codes = unique(deposits.msa_code);
MM = max(size(msa_codes_to_est));
dates = unique(deposits.date);
date_num = datenum(dates);
toc;

%% Loop over MSA markets
% Start a pool of 32 cores:
delete(gcp);
poolobj = parpool('local',32);    
for mm = 1:MM
msa_select = msa_codes_to_est(mm);
% estimates are based on the offer rates for 12-month CD
data_msa = deposits(deposits.msa_code==msa_select,{'id_rssd_hd_off','date','y','m' 'apy_cd12'});
dates_msa = unique(data_msa.date);
data_msa.date_num = datenum(data_msa.date);
TT = max(size(dates_msa));

%% Loop over all dates
clear res_est Rvec d t init R_t_est Rvec_est
disp('MSA area:');
disp(msa_code.msa_name(find(msa_code.msa_code==msa_select)));
res_est=[];
for t=1:TT;
    % select date and subset data for that date
    tic;
    d = dates_msa(t);
    disp(['Start new date:',d])
    data_msa_t = data_msa(data_msa.date_num==datenum(d),:);
    % rate information
    Rvec = str2double([data_msa_t.apy_cd12]);
    Rvec=sort(Rvec(:)); % sort in assending order
    Rvec = 1+Rvec/100; % normalize
    % delete NaN observations
    Rvec = Rvec(~isnan(Rvec));
    Rvec = Rvec(find(Rvec>0));
    R_min=min(Rvec); % lower bound
    R_max=max(Rvec); % upper bound
    N = max(size(Rvec)); % number of banks with non zero rates
    disp('Number of banks:');
    disp(N);
    %%---------------------------------------------------------------------
    if N>20 % only if there are 20 banks in the market
    % fixed parameters
    betta = 0.99;
    tau =1; % one year maturity for 12mo CD
    sigma = 1.276; % estimate from empirical equation in the paper
    
    %% Initial values for the parameter set  
        initq = rand(N,1);
        initq = initq/sum(initq);
 % Log-likelihood function-- evaluate at Rvec
    loglik_fixedsigma_z =@(z)loglik_sigmafixed_sym(z,sigma,betta,tau,Rvec, R_min, R_max);
    [finit,ginit] = loglik_fixedsigma_z(initq); % check value at inition point
    while isnan(sum(ginit)) || ~isfinite(sum(ginit))
        initq = rand(N,1);
        initq = initq/sum(initq);
        [finit,ginit] = loglik_fixedsigma_z(initq);
    end
    % lower and upper bounds
    lb= zeros(N,1)+2/10^6;
    [fval_lb, g_lb]=loglik_fixedsigma_z(lb);
    ub = ones(N,1)-2/10^6;
    [fval_ub, g_ub]=loglik_fixedsigma_z(ub);
 
    %% Optimization
    Aineq =[];
    bineq=[];
    % optimization options
    MaxIterations = 1000;
    MaxFunctionEvaluations = 10000;
    
    opts_fmincon=optimoptions(@fmincon,'Algorithm', 'interior-point',...
        'Display','off','Diagnostics','off' ,...
        'StepTolerance',1e-8,'OptimalityTolerance',1e-8,'ConstraintTolerance',1e-8,...
        'MaxIterations',MaxIterations,'MaxFunctionEvaluations',MaxFunctionEvaluations,...
        'UseParallel',1,...
        'CheckGradients', false, 'SpecifyObjectiveGradient',true,...
        'PlotFcns', []);
    
    problem = createOptimProblem('fmincon','objective', loglik_fixedsigma_z,...
        'x0', initq, ... % starting point
        'Aineq', Aineq, 'bineq',  bineq,... % inequality linear constraints
        'Aeq',  [], 'beq', [],  ... % equality constraints
        'nonlcon', [],... % non-linear constraints
        'lb', lb, 'ub', ub, 'options', opts_fmincon);
    
    % Run optimization
    tic;
    % Run global optimization
    ms = MultiStart('UseParallel', 1, 'StartPointsToRun','bounds');
    [ms_est,fval,est_exit,ms_out,~] = run(ms,problem, 64);
    qvec_est = ms_est;
    qvec_est = qvec_est/sum(qvec_est);
  

    %------------------------------------------------------------------
    %% compute search cost distribution
    %------------------------------------------------------------------
    disp('Compute delta_k and F_xi(delta_k)');
    delta_k = zeros(max(size(qvec_est)),1);
    qvec_rev= fliplr(qvec_est')';
    F_xi = cumsum(qvec_rev);
    parfor kk = 1:N
        delta_k(kk) = Delta_k(qvec_est,sigma,R_min,R_max,betta,tau,kk);
    end
    xi_vec = fliplr(delta_k')';
    
    %-------------------------------------------------------------------
    %% compute implied offer rate distribution
    %-------------------------------------------------------------------
    % Kolmogorov-Smirnov test
    % Estimate for the marginal cost of funds
    R_t_est = R_tilde(qvec_est,sigma,R_min,R_max,betta, tau);
    z_grid = linspace(0,1,1000);
    Rvec_est = R_z(z_grid,qvec_est,R_min, R_max,R_t_est, sigma, betta, tau);
    [ks_h,ks_pvalue,ks_stat]=kstest2(Rvec,Rvec_est);
    t_est = toc;
    
    if est_exit >0;
        res_est(t).msa = msa_select;
        res_est(t).sigma = sigma;
        res_est(t).qvec_est = qvec_est;
        res_est(t).q1 = qvec_est(1);
        res_est(t).qN = qvec_est(end);
        res_est(t).delta_k = delta_k;
        res_est(t).xi_vec = xi_vec;
        res_est(t).F_xi = F_xi;
        res_est(t).ks_pvalue = ks_pvalue;
        res_est(t).R_t = R_t_est;
        res_est(t).R_max = R_max;
        res_est(t).R_min = R_min;
        res_est(t).R_sd = std(Rvec);
        res_est(t).date = datestr(d);
        res_est(t).nbanks = N;
        res_est(t).exit = est_exit;
        res_est(t).fval = fval;
        res_est(t).t_est = t_est/60;
    end
    
    % print status
    display('----------------------------------------------------------')
    display('Market:')
    disp(msa_code.msa_name(find(msa_code.msa_code==msa_select)))
    disp(msa_select);
    disp(datestr(d))
    disp('Number of banks');
    disp(N);
    disp('Marginal cost of funds:')
    disp(R_t_est);
    disp('R_max:')
    disp(R_max);
    disp('R_min:')
    disp(R_min);
    display('Exit flag')
    disp(est_exit);
    display('Log-likelohood value')
    disp(fval);   
    display('Estimate of q1:')
    disp(qvec_est(1))
    display('Estimate of q_N:')
    disp(qvec_est(end))
    disp('Kolmogorov-Smirnov test statistic pvalue:')
    disp(ks_pvalue);
    display('Time to estimate (min):')
    disp(t_est/60);
    display('----------------------------------------------------------')
    end % check if at least 20 banks
end % end loop over dates
% Save results by MSA area
cd(results_dir);
save(['results_msa_',num2str(msa_select),'_',datestr(now,'dd_mmm_yyyy'),'.mat'], 'res_est');

end % end loop over MSA
%-------------------------------------------------------------------
delete(gcp); % stop processors
%-------------------------------------------------------------------
