# ----------------------------------------------------------------------------->
# Author: Vladimir Yankov
# Description: additional functions for analysis
# File: functions.R
# ----------------------------------------------------------------------------->
# Load packages----
detachAllPackages <- function() {

  basic.packages <- c("package:stats","package:graphics","package:grDevices",
                      "package:utils","package:datasets","package:methods",
                      "package:base")

  package.list <- search()[ifelse(unlist(gregexpr("package:",search()))==1,TRUE,FALSE)]

  package.list <- setdiff(package.list,basic.packages)

  if (length(package.list)>0)  for (package in package.list) detach(package, character.only=TRUE)

}
detachAllPackages()
libraries  = unique(c("foreign", "iterators", "foreach", "snow", "doSNOW",
                      "xlsx","haven","RPostgreSQL","RODBC","frb","latex2exp",
                      # multi-level panel regression (lfe)
                      "igraph","lfe","stargazer", "plm","lmtest",
                      "boot","bootstrap","sampling",
                      "fwildclusterboot","ClusterBootstrap","magrittr",
                      #"glmnet",
                      #"netprobitFE",
                      #"ivprobit",
                      #"ivreg","effects","sem",
                      # parallel computation
                      "doParallel","parallel",
                      # binary choice model with fixed effects
                      #"bife",
                      # geographic function
                      "maps","choroplethr", "choroplethrMaps",
                      "lubridate", "data.table", "zoo","dplyr"))
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}
# ----------------------------------------------------------------------------->
# Custom functions----
# ----------------------------------------------------------------------------->
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}

as.character.factor <- function(x){as.character(levels(x))[x]}

hhi <-function(x, normalize = T){
  n =length(x)
  xtotal =sum(x,na.rm=T)
  s = x/xtotal
  h = sum(s^2,na.rm=T)

  if(normalize ==T){
  h = (h-1/n)/(1-1/n)
  if(n==1){h=1}
  }

  return(h)
}

c1 <-function(x){
  n =length(x)
  xtotal =sum(x,na.rm=T)
  s = x/xtotal
  c = max(s,na.rm=T)
  c[is.infinite(c)]=0
  return(c)
}

c3 <-function(x){
  n =length(x)
  xtotal =sum(x,na.rm=T)
  s = x/xtotal
  s = s[order(-s)] # order in descending order
  c = sum(s[1:3],na.rm=T)
  c[is.infinite(c)]=0
  return(c)
}

sum.stats <-function(x, k =2){
  x <- outliers_remove(x)
  stats <- as.numeric(quantile(x,probs = c(0.05,0.25,0.5, 0.75, 0.95),
                               na.rm=T))
  stats <- c(c(mean(x,na.rm=T), sd(x,na.rm=T)),stats)
  stats<- round(stats, digits = k)
  names(stats) = c("mean","s.d.","5","25","50","75","95")
  return(stats)
}

sum.stats.99 <-function(x, k =2){
  x <- outliers_remove(x)
  stats <- as.numeric(quantile(x,probs = c(0.005,0.01,0.025,0.05,0.25,0.5, 0.75,0.95,0.975, 0.99, 0.995),
                               na.rm=T))
  stats <- c(c(mean(x,na.rm=T), sd(x,na.rm=T)),stats)
  stats<- round(stats, digits = k)
  return(stats)
}

growth.rate <- function(x,n=1){
  d_x = c(rep(NA,n),diff(x,n))/lag(x,n)
  d_x[which(is.infinite(d_x) |
              d_x > 5 | d_x < -5 )]=NA
  return(d_x)
}

log_growth<- function(x,k=1){
  # k lags
  y = 100*c(rep(NA,k),diff(log(x),lag=k))
  return(y)
}

sum.stats.iqr <-function(x, k =2){
  stats <- as.numeric(quantile(x,probs = c(0.25,0.5, 0.75),
                               na.rm=T))
  stats <- c(c(mean(x,na.rm=T), sd(x,na.rm=T)),stats)
  stats<- round(stats, digits = k)
  return(stats)
}


sum.stats.99 <-function(x, k =2){
  x <- outliers_remove(x)
  stats <- as.numeric(quantile(x,probs = c(0,0.01,0.05,0.25,0.5, 0.75,0.95, 0.99, 1),
                               na.rm=T))
  stats <- c(c(mean(x,na.rm=T), sd(x,na.rm=T)),stats)
  stats<- round(stats, digits = k)
  names(stats) = c("mean","std","min","p1","p5","p25","p50","p75","p95","p99","max")
  return(stats)
}

outliers_remove <-function(x,alpha_l=0.01, alpha_u=0.99){
  x[is.infinite(x)]=NA
  ul_limits <- quantile(x,probs = c(alpha_l,alpha_u),na.rm=T)
  x[which(x< ul_limits[1] | x > ul_limits[2] | is.infinite(x))]=NA
  y = x
  return(y)
}


reg_specifications <- function(y,x,fe_vars,cluster_vars,reg_data,
                               weights = NULL,
                               iv = "0",
                               subsample = NULL){
  ## subsample the data
  if(!is.null(subsample)){
    reg_data = reg_data[reg_data$year %in% subsample,]
  }

  reg_formula <-  as.formula(paste(y,"~",
                                   paste(x, collapse="+"),
                                   "|",
                                   paste(fe_vars, collapse = "+"),
                                   "|",
                                   iv,
                                   "|",
                                   paste(cluster_vars, collapse = "+")
  ))
  print("Regression specification:")
  print(reg_formula)

  reg_run <- felm(formula =reg_formula,
                  na.action = na.exclude,
                  cmethod ='reghdfe',
                  weights = weights,
                  data = reg_data)
  return(reg_run)
}


reg_formula <- function(y,x,fe_vars = "0",iv ="0",cluster_vars="0"){
  formula <- as.formula(paste(y,"~",
                              paste(x, collapse="+"),
                              "|",
                              paste(fe_vars, collapse = "+"),
                              "|",
                              iv,
                              "|",
                              paste(cluster_vars, collapse = "+")
  ))
  print(formula)
  return(formula)
}




























# ----------------------------------------------------------------------------->
