# ----------------------------------------------------------------------------->
# Author: Vladimir Yankov
# Description: Set of functions used in the structural estimation
# ----------------------------------------------------------------------------->

# ----------------------------------------------------------------------------->
# Fixed parameters
tau = 1
beta = 0.99
sigma = 1.27
# ----------------------------------------------------------------------------->
# marginal propensity to consume----
# ----------------------------------------------------------------------------->
mpc = function(R){
  if(sigma==1){
    hh = rep(1/(1+beta^tau), length(R))
    return(hh)
  }
  if(sigma!=1){
    return(1/(1+beta^(tau*sigma)*R^(sigma-1)))
  }
}
# ----------------------------------------------------------------------------->
# profit function----
# ----------------------------------------------------------------------------->
psi = function(R,R_tilde){
  return( (R_tilde - R)*(1-mpc(R)))
}
# psi_prime -- first derivative of psi
psi_prime = function(R,R_tilde){
  y = -(1-mpc(R))*(1+(1-sigma)*(R_tilde-R)*mpc(R)/R)
  return(y)
}
# inverse of the profit function -- R = inv_psi(psi)
psi_inv =function(pi,R_tilde){
  ff = function(R){psi(R,R_tilde)-pi}
  ff_inv = uniroot(ff,lower = 1, upper = 2*R_tilde, maxiter = 5000)$root
  return(ff_inv)
}


# ----------------------------------------------------------------------------->
# marginal utility of wealth----
# ----------------------------------------------------------------------------->
phi = function(R){
  if(sigma !=1){
    phi = mpc(R)^(1/(1-sigma))
  }
  if(sigma ==1){
    phi = log(mpc(R)) + (beta^tau)*log((1-mpc(R))*R)
  }
  return(phi)
}

## first derivative of the marginal utility of wealth
phi_prime = function(R){
  if(sigma !=1){
    phi_prime = phi(R)*(1-mpc(R))/R
    return(phi_prime)
  }
}
# ----------------------------------------------------------------------------->
# Offer rate distribution R(z) = inv(F(R))-----
# ----------------------------------------------------------------------------->
R_z = function(z, delta, R_min, R_tilde, Fxi_fit){
  N_1 = length(delta) # N-1
  N = N_1 + 1         # number of banks
  denominator = foreach(k=1:N, .combine = "sum")%do%{
    if(k>1 & k<N){
      term_k = k*z^(k-1)*(Fxi_fit(delta[k-1]) - Fxi_fit(delta[k]))
    }
    if(k==1){
      term_k = 1-Fxi_fit(delta[1])
    }
    if(k==N){
      term_k =  k*z^(k-1)*Fxi_fit(delta[N_1])
    }
    return(term_k)
  }
  pi = psi(R_min, R_tilde)*(1-Fxi_fit(delta[1]))/denominator
  R = psi_inv(pi,R_tilde)
  return(R)
}
R_z = Vectorize(R_z, vectorize.args = "z") # vectorize the function R_z


# maximal rate-----
R_max_fn.solve = function(qvec,R_tilde,R_min){
  N = length(qvec)
  qsum = foreach(k = 1:N,.combine = "sum")%do%{
    return(k*qvec[k])
  }
  q_ratio = qvec[1]/qsum
  R_max.sovle = function(x){
    h_ratio = (1-mpc(R_min))/(1-mpc(x))
    f = (R_tilde - x)-(R_tilde-R_min)*q_ratio*h_ratio
    return(f)
  }
  R.max <-uniroot(R_max.sovle,interval = c(1,1.5))$root
  return(R.max)
}

# equilibrium rate distribution----
F_R <-function(R,qvec,R_min,R_tilde){
  Fzero = function(z){
    N = length(qvec)
    q1 = qvec[1]
    sum_q_F = 0
    for(i in 1:N){
      sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
    }
    y = ( (R_tilde - R)*(1-mpc(R)))*sum_q_F - (R_tilde - R_min)*(1-mpc(R_min))*q1
    return(y)
  }
  Fzero = Vectorize(Fzero,vectorize.args = "z")
  R_max = R_max_fn.solve(qvec=qvec,R_tilde = R_tilde,R_min=R_min)

  FF = nleqslv(1/2,Fzero)$x
  if(R>R_max){FF=1}
  if(R<R_min){FF=0}
  return(FF)
}
F_R = Vectorize(F_R, vectorize.args = "R")

f_R = function(R,qvec,R_min,R_tilde){
  R_max = R_max_fn.solve(qvec,R_tilde,R_min)
  FF = F_R(R,qvec,R_min,R_tilde)
  q_sum_1 = 0
  q_sum_2 = 0
  N=length(qvec)
  for(i in 1:N){
    q_sum_1 = q_sum_1+i*qvec[i]*FF^(i-1)
    q_sum_2 = q_sum_2+i*(i-1)*qvec[i]*FF^(i-2)
  }
  ff = ((-1)*psi_prime(R,R_tilde)/psi(R,R_tilde))*q_sum_1/q_sum_2
  if(R>R_max){ff=0}
  if(R<R_min){ff=0}
  return(ff)
}
f_R = Vectorize(f_R, vectorize.args = "R")

# expected profit given F_R -----
pi_exp = function(R,qvec,R_min,R_max,R_tilde){
  FF = F_R(R,qvec, R_min, R_tilde)
  N = length(qvec)
  qsum_FR = foreach(k = 1:N,.combine = "sum")%do%{
    k*FF^(k-1)*qvec[k]
  }
  pi = psi(R,R_tilde)*(1/N)*qsum_FR
  return(pi)
}


# consumer surplus-----
cs_exp = function(delta,R_min,R_max,R_tilde,Fxi_fit){
  int_fun_cost = function(x){ (1- Fxi_fit(x))}
  qvec = qvec_delta(delta,Fxi_fit)
  N = length(qvec)
  registerDoParallel(4)
  cs_total <- foreach(k = 1:N, .combine = "sum")%dopar%{
    print(k)
    if(k==1){xi_k=0}
    if(k>1 & k < N){
      xi_k = integrate(int_fun2.w,lower = delta[k], upper =delta[k-1])$value
    }
    if(k==N){
      xi_k = integrate(int_fun2.w,lower = 0, upper =delta[k-1])$value
    }

    f_RR = function(R){f_R(R,qvec,R_min,R_tilde)}
    F_RR = function(R){F_R(R,qvec,R_min,R_tilde)}

    V_k = function(R){phi(R)*k*F_RR(R)^(k-1)*f_RR(R)}
    V_k_exp = integrate(V_k,lower=R_min,upper = R_max)$value - xi_k
    V_k_exp = qvec[k]*V_k_exp
    print(V_k_exp)
    return(V_k_exp)
  }
  stopImplicitCluster()
  cs_total = cs_total - phi(R_min)
  return(cs_total)
}


# Update delta-----
update_delta = function(delta, R_min,R_tilde,Fxi_fit){
  ## Update the equilibrium offer rate distribution
  R_zz = function(z){R_z(z,delta=delta, R_min=R_min,
                         R_tilde=R_tilde, Fxi_fit=Fxi_fit)}
  R_zz = Vectorize(R_zz, vectorize.args = "z") # vectorize the function R_z
  nbanks = length(delta)+1
  delta_k = function(k){
    f_k = function(z){return(phi(R_zz(z))*((k+1)*z - k)*z^(k-1))}
    f_k = Vectorize(f_k)
    Delta_k = integrate(f_k, 0,1)$value
  }
  ## update the equilibrium marginal values of information
  registerDoParallel(4)
  deltak_new = foreach(k=1:(nbanks-1), .combine ="c")%dopar%{
    print(k)
    dd = delta_k(k)
    return(dd)}
  stopImplicitCluster()
  deltak_new = deltak_new[order(-deltak_new)] # note the parallel computing scrambles the order
  return(deltak_new)
} # end update_delta


qvec_k = function(k, delta){
  N_1 = length(delta)
  N = N_1+1

  if(k!=1 & k!=N){
    qk = Fxi_fit(delta[k-1]) - Fxi_fit(delta[k])
  }
  if(k==1){
    qk = 1-Fxi_fit(delta[1])
  }
  if(k==N){
    qk = Fxi_fit(delta[N_1])
  }
  return(qk)
}

## marginal value of information/search
delta_k = function(k){
  f_k = function(z){return(phi(R_z(z))*((k+1)*z - k)*z^(k-1))}
  f_k = Vectorize(f_k)
  Delta_k = integrate(f_k, 0,1)$value
}

qvec_delta = function(delta,Fxi_fit){
  n = length(delta)
  qvec = rep(0,(n+1))
  qvec[1] = 1-Fxi_fit(delta[1])
  if(n>2){
    for(i in 2:n){
      qvec[i] = Fxi_fit(delta[i-1])-Fxi_fit(delta[i])
    }
  }
  qvec[n+1] = 1-sum(qvec[1:n])
  return(qvec)
}

supDist <- function(x,y){return(max(abs(x-y)))}
myIterator <- function(xinit, f, eps = 1e-06, itmax = 100, verbose = FALSE,
                       ...) {
  xold <- xinit
  itel <- 1
  repeat {
    xnew <- f(xold, ...)
    if (verbose)
      cat("Iteration: ", formatC(itel, width = 3, format = "d"), "xold: ",
          formatC(xold, digits = 8, width = 12, format = "f"), "xnew: ",
          formatC(xnew, digits = 8, width = 12, format = "f"), "\n")
    if ((supDist(xold, xnew) < eps) || (itel == itmax)) {
      return(xnew)
    }
    xold <- xnew
    itel <- itel + 1
  }
}

inverse.fun <- function(f, lower, upper){
  return(function(y){
    uniroot(function(x){f(x) - y}, lower = lower, upper = upper, tol=1e-3)$root[1]
  })
}

