# ----------------------------------------------------------------------------->
# Author: Vladimir Yankov
# Description: Set of functions used in the structural estimation
# ----------------------------------------------------------------------------->



# ----------------------------------------------------------------------------->
# Structural model functions----
# ----------------------------------------------------------------------------->
# Fixed parameters
tau = 1
beta = 0.99
sigma = 1.27

# ----------------------------------------------------------------------------->
# marginal propensity to consume
# ----------------------------------------------------------------------------->

h = function(R, sigma, beta, tau){
  if(sigma==1){
    hh = rep(1/(1+beta^tau), length(R))
    return(hh)
  }
  if(sigma!=1){
    return(1/(1+beta^(tau*sigma)*R^(sigma-1)))
  }
}



# ----------------------------------------------------------------------------->
# profit function
# ----------------------------------------------------------------------------->
psi = function(R,R_tilde, sigma, beta, tau){
  return( (R_tilde - R)*(1-h(R, sigma, beta, tau)))
}
# psi_prime -- first derivative of psi
psi_prime = function(R,R_tilde, sigma, beta, tau){
  y = -(1-h(R, sigma, beta, tau))*(1+(1-sigma)*(R_tilde-R)*h(R, sigma, beta, tau)/R)
  return(y)
}


# inverse of the profit function -- R = inv_psi(psi)
psi_inv =function(pi,R_tilde, sigma, beta, tau){

  ff = function(R){psi(R,R_tilde, sigma, beta, tau)-pi}

  ff_inv = uniroot(ff,lower = 1, upper = R_tilde, maxiter = 5000)$root
  return(ff_inv)}

# ----------------------------------------------------------------------------->
# marginal utility of wealth
# ----------------------------------------------------------------------------->


phi = function(R, sigma, beta, tau){
  if(sigma !=1){
    phi = h(R, sigma, beta, tau)^(1/(1-sigma))
  }
  if(sigma ==1){
    phi = log(h(R, sigma, beta, tau)) + (beta^tau)*log((1-h(R, sigma, beta, tau))*R)
  }
  return(phi)
}

## first derivative of the marginal utility of wealth
phi_prime = function(R, sigma, beta, tau){
  if(sigma !=1){
    phi_prime = phi(R, sigma, beta, tau)*(1-h(R, sigma, beta, tau))/R
    return(phi_prime)
  }
}
# Offer rate distribution
R_z = function(z, delta, R_min, R_tilde, sigma, beta, tau, Fxi_fit){
  N_1 = length(delta) # N-1
  denominator = foreach(k=1:N_1, .combine = "sum")%do%{
    if(k!=1 & k!=N_1){
      term_k = k*z^(k-1)*(Fxi_fit(delta[k-1]) - Fxi_fit(delta[k]))
    }
    if(k==1){
      term_k = 1-Fxi_fit(delta[1])
    }
    if(k==N_1){
      term_k = Fxi_fit(delta[N_1])
    }
    return(term_k)
  }
  R = psi(R_min, R_tilde,sigma, beta,tau)*(1-Fxi_fit(delta[1]))/denominator
  R = psi_inv(R,R_tilde,sigma, beta,tau)
  return(R)
}

R_zz = function(z){R_z(z, delta=delta, R_min=R_min,
                       R_tilde=R_tilde,
                       sigma=sigma, beta=beta,
                       tau=tau, Fxi_fit=Fxi_fit)}
R_zz = Vectorize(R_zz, vectorize.args = "z") # vectorize the function R_z

# maximal rate
R_max_fn.solve = function(qvec,R_tilde,R_min){

N = length(qvec)
qsum = foreach(k = 1:N,.combine = "sum")%do%{
return(k*qvec[k])
}
q_ratio = qvec[1]/qsum
R_max.sovle = function(x){
h_ratio = (1-h(R_min,sigma,tau,beta))/(1-h(x,sigma,tau,beta))
f = (R_tilde - x)-(R_tilde-R_min)*q_ratio*h_ratio
return(f)
}
R.max <-uniroot(R_max.sovle,interval = c(1,1.5))$root
return(R.max)
}

# equilibrium rate distribution
Fzero = function(z,R,qvec,R_min,R_tilde,sigma,beta,tau){
  N = length(qvec)
  q1 = qvec[1]
  sum_q_F = 0
  for(i in 1:N){
    sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
  }
  y = ( (R_tilde - R)*(1-h(R,sigma,beta,tau)))*sum_q_F - (R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*q1
  return(y)
}

F_R <-function(R){
  FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}
  FF = nleqslv(1/2,FFzero)$x
  if(R>R_max){FF=1}
  if(R<R_min){FF=0}
  return(FF)
}
F_R = Vectorize(F_R)

f_R = function(R){
  FF = F_R(R)
  q_sum_1 = 0
  q_sum_2 = 0
  N=length(qvec)
  for(i in 1:N){
    q_sum_1 = q_sum_1+i*qvec[i]*FF^(i-1)
    q_sum_2 = q_sum_2+i*(i-1)*qvec[i]*FF^(i-2)
  }
  ff = ((-1)*psi_prime(R,R_tilde, sigma, beta, tau)/psi(R,R_tilde, sigma, beta, tau))*q_sum_1/q_sum_2
  if(R>R_max){ff=0}
  if(R<R_min){ff=0}
  return(ff)
}
f_R = Vectorize(f_R)

V_k = function(k){
  int = function(R){
    return(phi(R,sigma, beta, tau)*k*F_R(R)^(k-1)*f_R(R))
  }
  VV = integrate(int,R_min,R_max)$value
  return(VV)
}
V_k = Vectorize(V_k)



Fzero = function(z,R,qvec,R_min,R_tilde,sigma,beta,tau){
                                      N = length(qvec)
                                      q1 = qvec[1]
                                      sum_q_F = 0
                                      for(i in 1:N){
                                        sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
                                      }
                                      y = ( (R_tilde - R)*(1-h(R,sigma,beta,tau)))*sum_q_F - (R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*q1
                                      return(y)
}

                                    F_R <-function(R){
                                      FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}
                                      FF = nleqslv(1/2,FFzero)$x
                                      if(R>R_max){FF=1}
                                      if(R<R_min){FF=0}
                                      return(FF)
                                    }
                                    F_R = Vectorize(F_R)

F_R.solve <-function(R,qvec,R_tilde,R_min){
R_max =R_max_fn.solve(qvec,R_tilde,R_min)
f  = function(x){
  N = length(qvec)
  rhs = ((R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*qvec[1])
  lhs = foreach(k = 1:N,.combine = "sum")%do%{
    return(k*qvec[k]*x^(k-1)*((R_tilde-R)*(1-h(R,sigma,beta,tau))))
  }
y = lhs - rhs
return(y)}
if(R > R_min & R < R_max & R < R_tilde){
F_R <-uniroot(f,interval = c(0,2*R_tilde), extendInt = "no")$root
}
if(R >= R_max){F_R = 1}
if(R <= R_min){F_R = 0}
if(F_R >1 | R >= R_max | R>=R_tilde){F_R =1}
return(F_R)
}


R.sd <- function(R,qvec,R_tilde,R_min){
R_max =R_max_fn.solve(qvec,R_tilde,R_min)
F_R = function(x){y = F_R.solve(x,qvec,R_tilde, R_min)}
int_fun1 = function(x){ x*(1- F_R(x))}
int_fun2 = function(x){(1- F_R(x))}

R_sd = sqrt(2*integrate(int_fun1,lower = R_min, upper =10*R_max)$value-
             integrate(int_fun2,lower = R_max, upper = 10*R_max)$value^2)
return(R_sd)
}



## marginal value of information/search
delta_k = function(k,R_zz){
  f_k = function(z){return(phi(R_zz(z), sigma, beta,tau)*((k+1)*z - k)*z^(k-1))}
  f_k = Vectorize(f_k)
  Delta_k = integrate(f_k, 0,1)$value

}

qvec_delta = function(delta,Fxi_fit){
  n = length(delta)
  qvec = rep(0,(n+1))
  qvec[1] = 1-Fxi_fit(delta[1])
  if(n>2){
  for(i in 2:n){
    qvec[i] = Fxi_fit(delta[i-1])-Fxi_fit(delta[i])
  }
  }
  qvec[n+1] = 1-sum(qvec[1:n])
return(qvec)
}





## updating function delta

update_delta = function(delta, R_min,R_tilde, sigma, beta,tau,Fxi_fit){

  ## Update the equilibrium offer rate distribution
  R_zz = function(z){R_z(z, delta=delta, R_min=R_min,
                         R_tilde=R_tilde,
                         sigma=sigma, beta=beta,
                         tau=tau, Fxi_fit=Fxi_fit)}
  R_zz = Vectorize(R_zz, vectorize.args = "z") # vectorize the function R_z
  nbanks = length(delta)+1
  ## update the equilibrium marginal values of information
  registerDoParallel(4)
  deltak_new = foreach(k=1:(nbanks-1), .combine ="c")%dopar%{
    print(k)
    dd = delta_k(k,R_zz=R_zz)
    return(dd)}
  stopImplicitCluster()
  deltak_new = deltak_new[order(-deltak_new)] # note the parallel computing scrambles the order
  return(deltak_new)
} # end update_delta


qvec_k = function(k, delta){
  N_1 = length(delta)
  N = N_1+1

  if(k!=1 & k!=N){
    qk = Fxi_fit(delta[k-1]) - Fxi_fit(delta[k])
  }
  if(k==1){
    qk = 1-Fxi_fit(delta[1])
  }
  if(k==N){
    qk = Fxi_fit(delta[N_1])
  }
  return(qk)
}



# ----------------------------------------------------------------------------->
# Summary statistics----
# ----------------------------------------------------------------------------->

Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    min = min(x,na.rm = T),
    max = max(x,na.rm = T))
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}


